<?php


namespace Schaffen\Core\Visitation\Managers;


use Freelabois\LaravelQuickstart\Extendables\ManipulationManager;
use Illuminate\Support\Facades\DB;
use Schaffen\Core\Form\Models\Form;
use Schaffen\Core\Form\Models\FormAssociation;
use Schaffen\Core\Form\Repositories\FormAssociationRepository;
use Schaffen\Core\Form\Repositories\FormGroupingRepository;
use Schaffen\Core\Form\Repositories\FormRepository;
use Schaffen\Core\Visitation\Models\VisitationReason;
use Schaffen\Core\Visitation\Repository\VisitationReasonRepository;

class VisitationReasonManager extends ManipulationManager
{
    private $form_repository;
    private $form_association_repository;

    public function __construct(VisitationReasonRepository $repository)
    {
        $this->form_repository = app()->make(FormRepository::class);
        $this->form_association_repository = app()->make( FormAssociationRepository::class);
        parent::__construct($repository);
    }

    public function storeOrUpdate($values, int $id = null, $relations = [])
    {
        if(empty($id)){
            $values['mov_status'] = VisitationReason::STATUS_ACTIVE;
        }
        $res = null;
        DB::transaction(function () use ($values, $id, $relations, &$res){
            $res = parent::storeOrUpdate($values, $id, $relations);
            if(isset($values['associate']) && count($values['associate']) > 0){
                $res->association()->delete();

                $form_grouping_repository = app()->make(FormGroupingRepository::class);

                foreach ($values['associate'] as $item){
                    if($item["type"] == 'group'){
                        $group = $form_grouping_repository->find($item['id'], ["forms"]);
                        foreach ($group->forms as $form){
                            $this->associateForm($res->mov_codigo, $form);
                        }
                    }
                    else if($item["type"] == 'form'){
                        $this->associateForm($res->mov_codigo, $item["id"]);
                    }
                }
                $res->load('forms');
            }
        });
        return $res;
    }

    private function associateForm($mov_codigo, $form_id){
        if(is_int($form_id))
            $form_db = $this->form_repository->find($form_id);
        else if($form_id instanceof Form){
            $form_db = $form_id;
            $form_id = $form_id->form_codigo;
        }

        $this->form_association_repository->firstOrNew(
            [
                "foa_formeavel_tipo" => FormAssociation::VISITATION_REASON,
                "foa_formeavel_codigo" => $mov_codigo,
                "foa_fk_form_codigo" => $form_id
            ],[], [["name" => 'form', "model" => $form_db]]
        );
    }
}